@@ -20,7 +20,8 @@
 #' \donttest{
 #' # Downloading data
 #' covid.df2 <- get_covid(year=2020, month=5, vars=c("C001","C002"),
-#'                        labels=TRUE, deflator=TRUE, design=FALSE, reload=TRUE, savedir=tempdir())
+#'                        labels=TRUE, deflator=TRUE, design=FALSE,
+#'                        reload=TRUE, curlopts=list(), savedir=tempdir())
 #' covid.svy2 <- covid_design(data_covid=covid.df2)
 #' # Calculating proportion of people temporarily away from work
 #' if (!is.null(covid.svy2)) survey::svymean(x=~C002, design=covid.svy2, na.rm=TRUE)}
@@ -37,12 +38,12 @@ covid_design <- function(data_covid) {
       data_posterior <- survey::postStratify(design=data_prior, strata=~posest, population=popc.types)
     }
     else {
-      message("Weight variables required for sample design are missing.")
+      message("Weight variables required for sample design are missing.\n")
       data_posterior <- data_covid
     }
   }
   else {
-    message("The microdata object is not of the tibble class or sample design was already defined for microdata, so applying another design is not possible.")
+    message("The microdata object is not of the tibble class or sample design was already defined for microdata, so applying another design is not possible.\n")
     data_posterior <- data_covid
   }
   return(data_posterior)
